package cairn

  1. Overview
  2. Docs

Module Parsing.PSource

Technical module containing the signature needed and provided by Cairn, in order to make them generic for the type of value returned by the parser.

Parameters

module T : sig ... end

Signature

Sourcemodule type parser = sig ... end

Signature of a module generated with menhir. Must be generated with options --table --inspection --cmly to work properly, as the inspection API is used, and the .cmly file is analysed.

Sourcemodule type parser_logger = sig ... end

Signature of the modules produced by this library. Common to Make and MakeWithDefaultMessage. Contains functions to get a log of the execution of the parser, as long with a graphical explorer of this log.