Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
parser_logger.ParserThe type of value that is produced by the parser. For the generated parser to be usable, it is advised to render this type visible.
val error_strategy : error_strategyIf Stop, the parser will stop at the first error encountered. If PopFirst, it will instead pop the stack until a terminal or non-terminal with attributes backup set in the grammar, and then ignores all tokens until the first that can be shifted and resume parser there -- not ideal, but the only way it seems to be possible.
module Parser : sig ... endModule generated with menhir. Must be generated with options --table --inspection --cmly to work properly, as the inspection API is used, and the .cmly file is analysed.
module Lexer : sig ... endLexer generated by menhir.