package cairn

  1. Overview
  2. Docs

Module type Parsing.parser_messagesSource

Signature matching the module generated by menhir with option --compile-errors. Used for error displaying. Will work if provided with a dummy message function instead (but less informative).

val message : int -> string